@@ -1,5 +1,5 @@
 /* code generation
-   Copyright (C) 2003, 2004
+   Copyright (C) 2003, 2004, 2005
    Craig Franklin
 
 This file is part of gputils.
@@ -22,6 +22,6 @@ Boston, MA 02111-1307, USA.  */
 #ifndef __CODEGEN14_H__
 #define __CODEGEN14_H__
 
-extern struct function_pointer_struct codegen14_func;
+extern target_type pic14;
 
 #endif
