@@ -1,5 +1,5 @@
 /* Common definitions for gpal
-   Copyright (C) 2003, 2004
+   Copyright (C) 2003, 2004, 2005
    Craig Franklin
 
 This file is part of gputils.
@@ -31,6 +31,7 @@ Boston, MA 02111-1307, USA.  */
 #endif
 
 #include "tree.h"
+#include "type.h"
 #include "symbol.h"
 
 extern struct gpal_state {
@@ -38,6 +39,8 @@ extern struct gpal_state {
   gp_boolean no_link;			/* compile and assemble, but don't link */
   gp_boolean archive;			/* compile and assemble, then link */
   gp_boolean delete_temps;		/* delete temporary files */
+  gp_boolean use_absolute_path;		/* Use absolute path on supported OS */
+  gp_boolean make_deps;			/* Output a dependency file */
   char *options;			/* extra link or lib options */
   struct {
     int level;
@@ -59,30 +62,31 @@ extern struct gpal_state {
   enum proc_class class;      		/* Processor class */
   struct px *processor_info;    	/* Processor identifiers (e.g. name) */
   gp_boolean processor_chosen;		/* Nonzero after processor-specific init */
-  enum size_tag pointer_size;		/* pointer size */
+  char *current_bank;			/* current bank */
+  char *current_ibank;			/* current ibank */
+  char *current_page;			/* current page */
   struct {				/* Processor data */
     char *code;      			/* code section name */
     int code_addr;			/* absolute address of code section */
     gp_boolean code_addr_valid;		/* is code address valid? */
-    char *udata;      			/* uninitialized data section name */
-    int udata_addr;			/* absolute address of udata section */
-    gp_boolean udata_addr_valid;	/* is udata address valid? */
+    char *data;      			/* data section name */
+    int data_addr;			/* absolute address of data section */
+    gp_boolean data_addr_valid;		/* is data address valid? */
   } section;
   struct symbol_table
     *modules,				/* Table of modules */
     *publics,				/* Table of publics */
     *global,				/* Base of Global symbols */
     *top,				/* Top of Global symbols */
-    *type;				/* Symbol Types */
+    *type,				/* Symbol Types */
+    *type_top;				/* Top of Symbol Types */
   tree *root;				/* start of tree */
   tree *module;				/* current tree being processed  */
   struct source_context *src;		/* Top of the stack of source files */
-  char *basefilename;			/* base filename */
-  char *srcfilename;			/* source filename */
   char *outfilename;			/* output filename */
-  char asmfilename[BUFSIZ];		/* asm output filename */
   struct {
     FILE *f;  				/* output assembly file */
+    FILE *d;  				/* output dependency file */
   } output;
 } state;
 
@@ -102,11 +106,11 @@ struct file_struct {
   gp_boolean is_link;		/* a file to add to the link list */
 };
 
-void add_entity(tree *node);
 int add_file(char *name,
              char *extension,
              gp_boolean is_temp,
              gp_boolean is_link);
 char *get_file_name(int id);
+void parse(char *file_name);
 
 #endif
