@@ -1,5 +1,5 @@
 /* optimize tree
-   Copyright (C) 2003, 2004
+   Copyright (C) 2003, 2004, 2005
    Craig Franklin
 
 This file is part of gputils.
@@ -23,7 +23,6 @@ Boston, MA 02111-1307, USA.  */
 
 #include "libgputils.h"
 #include "gpal.h"
-#include "symbol.h"
 #include "analyze.h"
 
 /* prototypes */
@@ -39,13 +38,21 @@ node_complexity(tree *node)
   int complexity = 0;
 
   switch(node->tag) {
-  case node_call:
+  case tag_attrib:
+    if ((ATTRIB_TYPE(node) == attrib_access) ||
+        (ATTRIB_TYPE(node) == attrib_address)) {
+      complexity = 2;    
+    } else { 
+      complexity = 1;
+    }
+    break;
+  case tag_call:
     complexity = 2;    
     break;
-  case node_constant:
+  case tag_constant:
     complexity = 1;
     break;
-  case node_symbol:
+  case tag_symbol:
     if (SYM_OFST(node)) {
       /* it is an array */
       complexity = 3;
@@ -53,10 +60,10 @@ node_complexity(tree *node)
       complexity = 2;    
     }
     break;
-  case node_unop:
+  case tag_unop:
     complexity = 1 + node_complexity(node->value.unop.p0);
     break;
-  case node_binop:
+  case tag_binop:
     complexity = 2 + 
                  node_complexity(node->value.binop.p0) +
                  node_complexity(node->value.binop.p1);
@@ -96,7 +103,7 @@ static gp_boolean
 is_constant(tree *p)
 {
 
-  if (p->tag == node_constant) {
+  if (p->tag == tag_constant) {
     return true;
   } else {
     return false;
@@ -108,7 +115,7 @@ static gp_boolean
 is_symbol(tree *p)
 {
 
-  if (p->tag == node_symbol) {
+  if (p->tag == tag_symbol) {
     return true;
   } else {
     return false;
@@ -120,7 +127,7 @@ static gp_boolean
 is_binop(tree *p)
 {
 
-  if (p->tag == node_binop) {
+  if (p->tag == tag_binop) {
     return true;
   } else {
     return false;
@@ -155,6 +162,7 @@ optimize_unop(tree *expr)
     /* remove the unop */
     expr = UNOP_ARG(expr);
     break;
+  case op_com:
   case op_not:
   case op_neg:
     if (is_constant(UNOP_ARG(expr))) {
@@ -196,9 +204,9 @@ optimize_binop(tree *expr)
   /* replace the operators that the code generator doesn't support */
   if ((BINOP_OP(expr) == op_gt) || (BINOP_OP(expr) == op_gte)) {
     if (BINOP_OP(expr) == op_gt) {
-      BINOP_OP(expr) = op_lte;
-    } else {
       BINOP_OP(expr) = op_lt;
+    } else {
+      BINOP_OP(expr) = op_lte;
     }
     temp = BINOP_RIGHT(expr);
     BINOP_RIGHT(expr) = BINOP_LEFT(expr);
@@ -450,21 +458,30 @@ opt_expr(tree *expr)
   struct variable *var;
 
   switch(expr->tag) {
-  case node_call:
-  case node_constant:
+  case tag_attrib:
+    if ((ATTRIB_TYPE(expr) != attrib_access) &&
+        (ATTRIB_TYPE(expr) != attrib_address))  {
+      /* convert all attribs into constants if possible */
+      expr = mk_constant(maybe_evaluate(expr));
+    }   
+    break;
+  case tag_call:
+  case tag_constant:
     /* do nothing */
     break;
-  case node_symbol:
+  case tag_symbol:
     var = get_global(SYM_NAME(expr));
     if (var->tag == sym_const) {
       /* remove all symbols that are constant */
       expr = mk_constant(var->value);
+    } else if (var->tag == sym_alias) {
+      expr = opt_expr(var->node);
     }
     break;
-  case node_unop:
+  case tag_unop:
     expr = optimize_unop(expr);
     break;
-  case node_binop:
+  case tag_binop:
     expr = optimize_binop(expr);
     break;
   default:
@@ -480,15 +497,25 @@ tree *
 optimize_expr(tree *expr)
 {
 
-  if (gp_debug_disable == 0) {
+  if (!gp_debug_disable) {
     gp_debug("====================================================");
     gp_debug("original:");
-    print_node(expr, 0);
+    print_node(expr, 0, false);
   }
+
   expr = opt_expr(expr);
-  if (gp_debug_disable == 0) {
+
+  if (!gp_debug_disable) {
     gp_debug("optimized:");
-    print_node(expr, 0);
+    print_node(expr, 0, false);
+  }
+
+  if (state.optimize.second_pass) {
+    expr = opt_expr(expr);
+    if (!gp_debug_disable) {
+      gp_debug("optimized (second pass):");
+      print_node(expr, 0, false);
+    }  
   }
 
   return expr;
@@ -504,25 +531,25 @@ optimize_unop_expr(struct variable *dest, tree *left, tree *right)
   tree *unop = NULL;
   struct variable *arg;
 
-  if ((right->tag == node_constant) && (right->value.constant == 0)) {
+  if ((right->tag == tag_constant) && (right->value.constant == 0)) {
     /* i = 0 */
     unop = mk_unop(op_clr, left);
-  } else if (right->tag == node_binop) {
+  } else if (right->tag == tag_binop) {
      binop_left = BINOP_LEFT(right);
      binop_right = BINOP_RIGHT(right);
      
      if (BINOP_OP(right) == op_add) {
-       if ((binop_left->tag == node_constant) && 
+       if ((binop_left->tag == tag_constant) && 
            (binop_left->value.constant == 1) &&
-           (binop_right->tag == node_symbol)) {
+           (binop_right->tag == tag_symbol)) {
          /* i = 1 + i ==> i++ */
          arg = get_global(SYM_NAME(binop_right));
          if (arg == dest) {
            unop = mk_unop(op_inc, left);
          }
-       } else if ((binop_right->tag == node_constant) && 
+       } else if ((binop_right->tag == tag_constant) && 
                   (binop_right->value.constant == 1) &&
-                  (binop_left->tag == node_symbol)) {
+                  (binop_left->tag == tag_symbol)) {
          /* i = i + 1 ==> i++ */
          arg = get_global(SYM_NAME(binop_left));
          if (arg == dest) {
@@ -531,9 +558,9 @@ optimize_unop_expr(struct variable *dest, tree *left, tree *right)
        }
      } else if (BINOP_OP(right) == op_sub) {
        /* i = i - 1 ==> i-- */
-       if ((binop_right->tag == node_constant) && 
+       if ((binop_right->tag == tag_constant) && 
            (binop_right->value.constant == 1) &&
-           (binop_left->tag == node_symbol)) {
+           (binop_left->tag == tag_symbol)) {
          arg = get_global(SYM_NAME(binop_left));
          if (arg == dest) {
            unop = mk_unop(op_dec, left);
@@ -546,13 +573,13 @@ optimize_unop_expr(struct variable *dest, tree *left, tree *right)
 
   /* i = i & 0xfe ==> bitclear(i, 0) */
 
-  if ((gp_debug_disable == 0) && (unop != NULL)) {
+  if ((!gp_debug_disable) && (unop != NULL)) {
     gp_debug("====================================================");
     gp_debug("original binop:");
-    print_node(left, 0);
-    print_node(right, 0);
+    print_node(left, 0, false);
+    print_node(right, 0, false);
     gp_debug("optimized unop:");
-    print_node(unop, 0);
+    print_node(unop, 0, false);
   }
   
   return unop;
