@@ -1,5 +1,5 @@
 /* parse tree
-   Copyright (C) 2003, 2004
+   Copyright (C) 2003, 2004, 2005
    Craig Franklin
 
 This file is part of gputils.
@@ -22,33 +22,32 @@ Boston, MA 02111-1307, USA.  */
 #ifndef __TREE_H__
 #define __TREE_H__
 
-/* FIXME: change name from node_unknown to tag_unknown */
-
 enum node_tag { 
-  node_unknown,
-  node_alias,
-  node_arg,
-  node_assembly,
-  node_body,
-  node_binop,
-  node_call,
-  node_constant, 
-  node_cond,
-  node_decl,
-  node_file,
-  node_func,  
-  node_goto,  
-  node_head,  
-  node_label,
-  node_loop,
-  node_pragma,
-  node_proc,
-  node_return,
-  node_string,
-  node_symbol, 
-  node_type,
-  node_unop,
-  node_with
+  tag_unknown,
+  tag_alias,
+  tag_arg,
+  tag_assembly,
+  tag_attrib,
+  tag_body,
+  tag_binop,
+  tag_call,
+  tag_constant,
+  tag_cond,
+  tag_decl,
+  tag_file,
+  tag_goto,  
+  tag_head,
+  tag_label,
+  tag_loop,
+  tag_pragma,
+  tag_record,
+  tag_return,
+  tag_string,
+  tag_subprogram,
+  tag_symbol,
+  tag_type,
+  tag_unop,
+  tag_with
 };
 
 enum node_op { 
@@ -95,7 +94,7 @@ enum node_storage {
   storage_unknown,
   storage_public,  /* data which is visible to other modules */
   storage_private, /* data which is not visible to other modules */
-  storage_local,   /* data which is local to current procedure */
+  storage_local,   /* data which is local to a subprogram */
   storage_near,    /* external data on the same page or bank with module */
   storage_far      /* external data which is on an unknown page or bank */
 };
@@ -103,8 +102,17 @@ enum node_storage {
 enum source_type {
   source_unknown,
   source_module,   /* a source file */
-  source_public,   /* the public file for the module being compiled */
-  source_with      /* the public file for a external module */ 
+  source_with      /* a public file */ 
+};
+
+enum attrib_type {
+  attrib_unknown,
+  attrib_access,
+  attrib_address,
+  attrib_first,
+  attrib_last,
+  attrib_range,
+  attrib_size
 };
 
 typedef struct node_struct tree;
@@ -114,7 +122,7 @@ typedef struct node_struct {
   union {
     struct {
       char *alias;
-      char *name;
+      tree *expr;
     } alias;
     struct {
       char *name;
@@ -122,6 +130,10 @@ typedef struct node_struct {
       char *type;
     } arg;
     char *assembly;
+    struct {
+      char *name;
+      enum attrib_type type;
+    } attrib;
     struct {
       tree *decl;
       tree *statements;
@@ -143,7 +155,8 @@ typedef struct node_struct {
     } cond;    
     struct {
       char *name;
-      gp_boolean constant;
+      gp_boolean is_volatile;
+      gp_boolean is_constant;
       char *type;
       tree *init;
       tree *addr;
@@ -153,13 +166,10 @@ typedef struct node_struct {
       char *name;
       enum source_type type;
       enum node_storage code_default;
-      enum node_storage udata_default;
+      enum node_storage data_default;
+      char *page_address;
+      char *bank_address;
     } file;
-    struct {
-      tree *head;
-      char *ret;
-      tree *body;
-    } func;
     struct {
       char *name;
     } _goto;
@@ -179,17 +189,23 @@ typedef struct node_struct {
     } loop;
     tree *pragma;
     struct {
-      tree *head;
-      tree *body;
-    } proc;
+      char *name;
+      tree *list;
+    } record;
     tree *ret;
     char *string;
+    struct {
+      tree *head;
+      char *ret;
+      tree *body;
+    } subprogram;
     struct {
       char *name;
       tree *offset;
     } symbol;
     struct {
       char *type;
+      gp_boolean is_access;
       tree *start;
       tree *end;
       tree *list;
@@ -215,10 +231,12 @@ typedef struct node_struct {
 } node;
 
 #define ALIAS_ALIAS(A)     (A)->value.alias.alias
-#define ALIAS_NAME(A)      (A)->value.alias.name
+#define ALIAS_EXPR(A)      (A)->value.alias.expr
 #define ARG_NAME(A)        (A)->value.arg.name
 #define ARG_DIR(A)         (A)->value.arg.dir
 #define ARG_TYPE(A)        (A)->value.arg.type
+#define ATTRIB_NAME(A)     (A)->value.attrib.name
+#define ATTRIB_TYPE(A)     (A)->value.attrib.type
 #define BODY_DECL(B)       (B)->value.body.decl
 #define BODY_STATEMENTS(B) (B)->value.body.statements
 #define BINOP_OP(B)        (B)->value.binop.op
@@ -230,7 +248,8 @@ typedef struct node_struct {
 #define COND_BODY(C)       (C)->value.cond.body
 #define COND_NEXT(C)       (C)->value.cond.next
 #define DECL_NAME(D)       (D)->value.decl.name
-#define DECL_CONSTANT(D)   (D)->value.decl.constant
+#define DECL_VOLATILE(D)   (D)->value.decl.is_volatile
+#define DECL_CONSTANT(D)   (D)->value.decl.is_constant
 #define DECL_TYPE(D)       (D)->value.decl.type
 #define DECL_INIT(D)       (D)->value.decl.init
 #define DECL_ADDR(D)       (D)->value.decl.addr
@@ -238,10 +257,9 @@ typedef struct node_struct {
 #define FILE_NAME(F)       (F)->value.file.name
 #define FILE_TYPE(F)       (F)->value.file.type
 #define FILE_CODE(F)       (F)->value.file.code_default
-#define FILE_UDATA(F)      (F)->value.file.udata_default
-#define FUNC_HEAD(F)       (F)->value.func.head
-#define FUNC_RET(F)        (F)->value.func.ret
-#define FUNC_BODY(F)       (F)->value.func.body
+#define FILE_DATA(F)       (F)->value.file.data_default
+#define FILE_CODE_ADDR(F)  (F)->value.file.page_address
+#define FILE_DATA_ADDR(F)  (F)->value.file.bank_address
 #define GOTO_NAME(G)       (G)->value._goto.name
 #define HEAD_NAME(H)       (H)->value.head.name
 #define HEAD_ARGS(H)       (H)->value.head.args
@@ -250,11 +268,15 @@ typedef struct node_struct {
 #define LOOP_EXIT(L)       (L)->value.loop.exit
 #define LOOP_INCR(L)       (L)->value.loop.incr
 #define LOOP_BODY(L)       (L)->value.loop.body
-#define PROC_HEAD(P)       (P)->value.proc.head
-#define PROC_BODY(P)       (P)->value.proc.body
+#define RECORD_NAME(R)     (R)->value.record.name 
+#define RECORD_LIST(R)     (R)->value.record.list
+#define SUB_HEAD(S)        (S)->value.subprogram.head
+#define SUB_RET(S)         (S)->value.subprogram.ret
+#define SUB_BODY(S)        (S)->value.subprogram.body
 #define SYM_NAME(T)        (T)->value.symbol.name
 #define SYM_OFST(T)        (T)->value.symbol.offset
 #define TYPE_TYPE(T)       (T)->value.type.type
+#define TYPE_ACCESS(T)     (T)->value.type.is_access
 #define TYPE_START(T)      (T)->value.type.start
 #define TYPE_END(T)        (T)->value.type.end
 #define TYPE_LIST(T)       (T)->value.type.list
@@ -271,31 +293,38 @@ void free_nodes(void);
 
 tree *mk_node(enum node_tag tag);
 
-tree *mk_alias(char *alias, char *name);
+tree *mk_alias(char *alias, tree *expr);
 tree *mk_arg(char *name, enum node_dir dir, char *type);
 tree *mk_assembly(char *assembly);
+tree *mk_attrib(char *name, enum attrib_type type);
 tree *mk_body(tree *decl, tree *statements);
 tree *mk_binop(enum node_op op, tree *p0, tree *p1);
 tree *mk_call(char *name, tree *args);
 tree *mk_constant(int value);
 tree *mk_cond(tree *cond, tree *body, tree *next);
 tree *mk_decl(char *name,
-              gp_boolean constant,
+              gp_boolean is_volatile,
+              gp_boolean is_constant,
               char *type,
               tree *init,
               tree *addr);
 tree *mk_file(tree *body, char *name, enum source_type type);
-tree *mk_func(tree *head, char *ret, tree *body);
 tree *mk_goto(char *name);
 tree *mk_head(char *name, tree *args);
 tree *mk_label(char *name);
 tree *mk_loop(tree *init, tree *exit, tree *incr, tree *body);
 tree *mk_pragma(tree *pragma);
-tree *mk_proc(tree *head, tree *body);
+tree *mk_record(char *name, tree *list);
 tree *mk_return(tree *ret);
 tree *mk_string(char *value);
+tree *mk_subprogram(tree *head, char *ret, tree *body);
 tree *mk_symbol(char *name, tree *offset);
-tree *mk_type(char *type, tree *start, tree *end, tree *list, char *of);
+tree *mk_type(char *type, 
+              gp_boolean is_access,
+              tree *start,
+              tree *end,
+              tree *list,
+              char *of);
 tree *mk_unop(enum node_op op, tree *p0);
 tree *mk_with(char *name);
 
@@ -304,6 +333,6 @@ tree *node_list(tree *head, tree *tail);
 char *find_node_name(tree *node);
 tree *find_node(tree *search, char *name, enum node_tag tag);
 
-void print_node(tree *node, int level);
+void print_node(tree *node, int level, gp_boolean print_list);
 
 #endif
