@@ -1,5 +1,5 @@
 /* Generate coff file
-   Copyright (C) 2002, 2003, 2004
+   Copyright (C) 2002, 2003, 2004, 2005
    Craig Franklin
 
 This file is part of gputils.
@@ -38,17 +38,17 @@ coff_init(void)
   }
 
   if (state.objfile == suppress) {
-    state.obj.enabled = 0;
+    state.obj.enabled = false;
     unlink(state.objfilename);
   } else {
     if (state.processor_chosen == 0) {
-      state.obj.enabled = 0;
+      state.obj.enabled = false;
     } else {
       state.obj.object = gp_coffgen_init();
       state.obj.object->filename = strdup(state.objfilename);
       state.obj.object->processor = state.processor;
       state.obj.object->class = state.device.class;
-      state.obj.enabled = 1;
+      state.obj.enabled = true;
     }
   }
   
@@ -132,7 +132,7 @@ _copy_config(void)
   int start;
   int stop;
   int word;
-  int found_break;
+  gp_boolean found_break;
   
   if (state.obj.section == NULL)
     return;
@@ -196,7 +196,7 @@ _copy_config(void)
     if(_16bit_core) {
       start = config_section->address >> 1;
       stop = IDLOC7 >> 1;
-      found_break = 0;
+      found_break = false;
       for (i = start; i <= stop; i++) {
         printf("address = %x\n", i);
         word = i_memory_get(state.c_memory, i);
@@ -207,7 +207,7 @@ _copy_config(void)
           i_memory_put(config_section->data, i, word);
           config_section->size += 2;    
         } else {
-          found_break = 1;
+          found_break = true;
         }
       }
       
@@ -400,20 +400,20 @@ coff_linenum(int emitted)
 /* Add a symbol to the coff symbol table.  The calling function must
    increment the global symbol number. */
 
-void
+gp_symbol_type *
 coff_add_sym(char *name, int value, enum gpasmValTypes type)
 {
-  gp_symbol_type *new;
+  gp_symbol_type *new = NULL;
   char message[BUFSIZ];
   int section_number = 0;
   int class = C_EXT; 
 
   if(!state.obj.enabled)
-    return;
+    return NULL;
 
   switch (type) {
   case gvt_extern:
-    section_number = 0;
+    section_number = N_UNDEF;
     class = C_EXT;
     break;
   case gvt_global:
@@ -428,8 +428,16 @@ coff_add_sym(char *name, int value, enum gpasmValTypes type)
     section_number = state.obj.section_num;
     class = C_LABEL;
     break;
+  case gvt_debug:
+    section_number = N_DEBUG;
+    class = C_NULL;
+    break;
+  case gvt_absolute:
+    section_number = N_ABS;
+    class = C_NULL;
+    break;
   default:
-    return;
+    return new;
   }
 
   new = gp_coffgen_findsymbol(state.obj.object, name);
@@ -445,7 +453,7 @@ coff_add_sym(char *name, int value, enum gpasmValTypes type)
     }
   }
 
-  if ((new != NULL) && (type != gvt_extern))  {
+  if ((new != NULL) && (type != gvt_extern) && (type != gvt_debug))  {
     snprintf(message, sizeof(message),
              "Duplicate label or redefining symbol that cannot be redefined. (%s)",
              name);    
@@ -460,7 +468,7 @@ coff_add_sym(char *name, int value, enum gpasmValTypes type)
     new->class          = class;
   }
  
-  return;
+  return new;
 }
 
 /* add a file symbol to the coff symbol table */
@@ -572,6 +580,65 @@ coff_add_nolistsym(void)
   return;
 }
 
+/* add a direct symbol to the coff symbol table */
+
+void
+coff_add_directsym(unsigned char command, char *string)
+{
+  gp_symbol_type *new = NULL;
+  gp_aux_type *new_aux;
+
+  state.obj.symbol_num += 2;
+
+  if(!state.obj.enabled)
+    return;
+
+  /* add .cod symbol */
+  new = gp_coffgen_addsymbol(state.obj.object);
+  new->name           = strdup(".direct");
+  new->value          = state.org << _16bit_core;
+  new->section_number = state.obj.section_num;
+  new->section        = state.obj.section;
+  new->type           = T_NULL;
+  new->class          = C_NULL;
+
+  new_aux = gp_coffgen_addaux(state.obj.object, new);
+  new_aux->type = AUX_DIRECT;
+  new_aux->_aux_symbol._aux_direct.command = command;
+  new_aux->_aux_symbol._aux_direct.string = strdup(string);
+ 
+  return;
+}
+
+/* add a cod symbol to the coff symbol table */
+
+void
+coff_add_identsym(char *string)
+{
+  gp_symbol_type *new = NULL;
+  gp_aux_type *new_aux;
+
+  state.obj.symbol_num += 2;
+
+  if(!state.obj.enabled)
+    return;
+
+  /* add .cod symbol */
+  new = gp_coffgen_addsymbol(state.obj.object);
+  new->name           = strdup(".ident");
+  new->value          = 0;
+  new->section_number = N_DEBUG;
+  new->section        = NULL;
+  new->type           = T_NULL;
+  new->class          = C_NULL;
+
+  new_aux = gp_coffgen_addaux(state.obj.object, new);
+  new_aux->type = AUX_IDENT;
+  new_aux->_aux_symbol._aux_ident.string = strdup(string);
+ 
+  return;
+}
+
 /* If the symbol is local, generate a modified name for the coff symbol 
    table. */
 
