@@ -1,5 +1,5 @@
 /* Implements directives, pseudo-ops and processor opcodes
-   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004
+   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005
    James Bowman, Craig Franklin
 
 This file is part of gputils.
@@ -435,11 +435,7 @@ static gpasmVal do_bankisel(gpasmVal r,
   if (enforce_arity(arity, 1)) {
     p = HEAD(parms);
     if (state.mode == absolute) {
-      if (p->tag != symbol) {
-        gperror(GPE_ILLEGAL_LABEL, NULL);
-      } else {
-        set_bankisel(maybe_evaluate(p));
-      }
+      set_bankisel(maybe_evaluate(p));
     } else {
       num_reloc = count_reloc(p);
  
@@ -471,17 +467,13 @@ static gpasmVal do_banksel(gpasmVal r,
   if (enforce_arity(arity, 1)) {
     p = HEAD(parms);
     if (state.mode == absolute) {
-      if (p->tag != symbol) {
-        gperror(GPE_ILLEGAL_LABEL, NULL);
-      } else {
-        address = maybe_evaluate(p);
-        bank = gp_processor_check_bank(state.device.class, address);
-        state.org += gp_processor_set_bank(state.device.class, 
-                                           state.processor_info->num_banks,
-                                           bank, 
-                                           state.i_memory, 
-                                           state.org);
-      }
+      address = maybe_evaluate(p);
+      bank = gp_processor_check_bank(state.device.class, address);
+      state.org += gp_processor_set_bank(state.device.class, 
+                                         state.processor_info->num_banks,
+                                         bank, 
+                                         state.i_memory, 
+                                         state.org);
     } else {
       num_reloc = count_reloc(p);
 
@@ -636,12 +628,12 @@ static gpasmVal do_config(gpasmVal r,
   state.lst.config_address = ca;
 
   if (state.mode == relocatable) {
-    if ((state.found_devid == 0) && ((ca == DEVID1) || (ca == DEVID2))) {
+    if ((!state.found_devid) && ((ca == DEVID1) || (ca == DEVID2))) {
       coff_new_section(".devid", ca >> _16bit_core, STYP_ABS | STYP_TEXT);
-      state.found_devid = 1;
-    } else if (state.found_config == 0) {
+      state.found_devid = true;
+    } else if (!state.found_config) {
       coff_new_section(".config", ca >> _16bit_core, STYP_ABS | STYP_TEXT);
-      state.found_config = 1;
+      state.found_config = true;
     }
   }
 
@@ -801,6 +793,261 @@ static gpasmVal do_de(gpasmVal r,
   return r;
 }
 
+static gpasmVal do_def(gpasmVal r,
+                       char *name,
+                       int arity,
+                       struct pnode *parms)
+{
+  struct pnode *p;
+  char *symbol_name = NULL;
+  gp_symbol_type *coff_symbol = NULL;
+  int shift = 0;
+  int eval;
+  int value = 0;
+  gp_boolean new_class = false;
+  int coff_class = C_NULL;
+  gp_boolean new_type = false;
+  int coff_type = T_NULL;
+  enum gpasmValTypes type = gvt_debug;
+
+  state.lst.line.linetype = dir;
+
+  if (state.mode == absolute) {
+    gperror(GPE_OBJECT_ONLY, NULL);
+  } else {
+    if (arity < 2) { 
+      enforce_arity(arity, 2);
+      return r;
+    }
+
+    /* the first argument is the symbol name */
+    p = HEAD(parms);
+    if (enforce_simple(p)) {
+      symbol_name = p->value.symbol;
+    } else {
+      return r;
+    }
+    parms = TAIL(parms);
+
+    if ((SECTION_FLAGS & STYP_TEXT) && _16bit_core) {
+      shift = 1;
+    }
+
+    /* update the properties */
+    for (; parms; parms = TAIL(parms)) {
+      p = HEAD(parms);
+      if ((p->tag == binop) &&
+	  (p->value.binop.op == '=')) {
+        if (enforce_simple(p->value.binop.p0)) {
+	  char *lhs;
+
+	  lhs = p->value.binop.p0->value.symbol;
+          if (strcasecmp(lhs, "value") == 0) {
+            value = maybe_evaluate(p->value.binop.p1);   
+    	  } else if (strcasecmp(lhs, "size") == 0) {
+            state.org += (maybe_evaluate(p->value.binop.p1) >> shift);   
+    	  } else if (strcasecmp(lhs, "type") == 0) {
+            eval = maybe_evaluate(p->value.binop.p1);   
+            if ((eval < 0) || (eval > 0xffff)) {
+              gperror(GPE_RANGE, NULL);
+            } else {
+              new_type = true;
+              coff_type = eval;   
+            }
+    	  } else if (strcasecmp(lhs, "class") == 0) {
+            eval = maybe_evaluate(p->value.binop.p1);   
+            if ((eval < -128) || (eval > 127)) {
+              gperror(GPE_RANGE, NULL);
+            } else {
+              new_class = true;
+              coff_class = eval;   
+            }
+          } else {
+            gperror(GPE_ILLEGAL_ARGU, NULL);
+          }
+        }
+      } else {
+        if (enforce_simple(p)) {
+	  if (strcasecmp(p->value.symbol, "absolute") == 0) {
+            type = gvt_absolute;
+            value = 0;   
+  	  } else if (strcasecmp(p->value.symbol, "debug") == 0) {
+            type = gvt_debug;
+            value = 0;   
+	  } else if (strcasecmp(p->value.symbol, "extern") == 0) {
+            type = gvt_extern;
+            value = 0;   
+  	  } else if (strcasecmp(p->value.symbol, "global") == 0) {
+            type = gvt_global;
+            value = state.org << shift;
+  	  } else if (strcasecmp(p->value.symbol, "static") == 0) {
+            type = gvt_static;
+            value = state.org << shift;
+  	  } else {
+            gperror(GPE_ILLEGAL_ARGU, NULL);
+          }
+        }
+      }
+    }
+  }
+
+  set_global(symbol_name, value, PERMANENT, type);
+
+  /* update the symbol with the values */
+  if ((state.pass == 2) && (new_class || new_type)) {
+    coff_symbol = gp_coffgen_findsymbol(state.obj.object, symbol_name);
+    assert(coff_symbol != NULL);
+    if (new_class)
+      coff_symbol->class = coff_class;
+      
+    if (new_type)
+      coff_symbol->type = coff_type;
+  }
+
+  return r;
+}
+
+static gpasmVal do_dim(gpasmVal r,
+                       char *name,
+                       int arity,
+                       struct pnode *parms)
+{
+  struct pnode *p;
+  char *symbol_name = NULL;
+  gp_symbol_type *coff_symbol = NULL;
+  int number_symbols;
+  gp_aux_type *aux_list;
+  struct sllist first_list;
+  struct sllist *list = &first_list;
+  struct sllist *previous;
+  int i;
+  int value;
+
+  state.lst.line.linetype = dir;
+
+  if (state.mode == absolute) {
+    gperror(GPE_OBJECT_ONLY, NULL);
+  } else {
+    if (arity < 3) { 
+      enforce_arity(arity, 1);
+      return r;
+    }
+    
+    /* the first argument is the symbol name */
+    p = HEAD(parms);
+    if (enforce_simple(p)) {
+      /* lookup the symbol */
+      symbol_name = p->value.symbol;
+      coff_symbol = gp_coffgen_findsymbol(state.obj.object, symbol_name);
+      if (coff_symbol == NULL) {
+        gperror(GPE_NOSYM, NULL);
+        return r;
+      }
+    } else {
+      return r;
+    }
+    parms = TAIL(parms);
+
+    /* the second argument must be the number of aux symbols */
+    p = HEAD(parms);
+    number_symbols = maybe_evaluate(p);
+    if ((number_symbols < 0) || (number_symbols > 127)) {
+      gperror(GPE_UNKNOWN, "number of auxiliary symbols must be less then 128 and positive"); 
+      return r;
+    }
+    state.obj.symbol_num += number_symbols;
+    parms = TAIL(parms);
+
+    /* create the symbols */
+    aux_list = gp_coffgen_blockaux(number_symbols);
+    coff_symbol->num_auxsym = number_symbols;
+    coff_symbol->aux_list = aux_list;
+
+    /* convert the arguments into a list of values */
+    list->next = NULL;
+    simplify_data(parms, list, 0);
+
+    /* write the data to the auxiliary symbols */
+    list = list->next;
+    i = 0;
+    while(list) {
+      value = maybe_evaluate(list->p);
+      if (value & (~0xff)) {
+        gperror(GPE_RANGE, NULL); 
+        return r;
+      }
+      
+      if (aux_list == NULL) {
+        gperror(GPE_UNKNOWN, "insufficent number of auxiliary symbols"); 
+        return r;
+      }
+      
+      if (i == SYMBOL_SIZE) {
+        i = 0;
+        aux_list = aux_list->next;      
+      } else {
+        aux_list->_aux_symbol.data[i++] = value;
+      }
+
+      previous = list;
+      list = list->next;
+      free(previous);
+    }
+  }
+
+  return r;
+}
+
+static gpasmVal do_direct(gpasmVal r,
+		          char *name,
+		          int arity,
+		          struct pnode *parms)
+{
+
+  struct pnode *p;
+  int value;
+  unsigned char direct_command = 0;
+  char *direct_string = NULL;
+
+  state.lst.line.linetype = dir;
+
+  if (state.mode == absolute) {
+    gperror(GPE_OBJECT_ONLY, NULL);
+  } else if (enforce_arity(arity, 2)) {
+    p = HEAD(parms);
+    value = maybe_evaluate(p);
+    if ((value < 0) || (value > 255)) {
+      gperror(GPE_RANGE, NULL);
+    } else {
+      direct_command = value;   
+    }
+
+    p= HEAD(TAIL(parms));
+    if (p->tag == string) {
+      if (strlen(p->value.string) < 255) {
+        direct_string = p->value.string;
+      } else {
+        gperror(GPE_UNKNOWN, "string must be less than 255 bytes long");
+      }
+    } else {
+      gperror(GPE_ILLEGAL_ARGU, NULL);
+    }  
+
+    if (direct_string == NULL) {
+      return r;
+    }
+
+    if (SECTION_FLAGS & STYP_TEXT) {
+      coff_add_directsym(direct_command, direct_string);
+    } else {
+      gperror(GPE_WRONG_SECTION, NULL);
+    }
+  
+  }
+
+  return r;
+}
+
 static gpasmVal do_dt(gpasmVal r,
 		      char *name,
 		      int arity,
@@ -936,6 +1183,26 @@ static gpasmVal do_endw(gpasmVal r,
   return r;
 }
 
+static gpasmVal do_eof(gpasmVal r,
+		       char *name,
+		       int arity,
+		       struct pnode *parms)
+{
+  state.lst.line.linetype = dir;
+
+  if (state.mode == absolute) {
+    gperror(GPE_OBJECT_ONLY, NULL);
+  } else if (enforce_arity(arity, 0)) {
+    if (state.debug_info) { 
+      coff_add_eofsym();
+    } else {
+      gpwarning(GPW_UNKNOWN, "directive ignored when debug info is disabled");
+    }
+  }
+
+  return r;
+}
+
 static gpasmVal do_equ(gpasmVal r,
 		       char *name,
 		       int arity,
@@ -1042,11 +1309,11 @@ static gpasmVal do_expand(gpasmVal r,
 		       struct pnode *parms)
 {
   state.lst.line.linetype = dir;
-  if (state.cmd_line.macro_expand == 1){
+  if (state.cmd_line.macro_expand) {
     gpmessage(GPM_SUPLIN, NULL);
   } else {
     if (enforce_arity(arity, 0)) {
-      state.lst.expand = 1;
+      state.lst.expand = true;
     }
   }
   return r;
@@ -1076,6 +1343,32 @@ static gpasmVal do_extern(gpasmVal r,
   return r;
 }
 
+static gpasmVal do_file(gpasmVal r,
+		       char *name,
+		       int arity,
+		       struct pnode *parms)
+{
+  struct pnode *p;
+  state.lst.line.linetype = dir;
+  
+  if (state.mode == absolute) {
+    gperror(GPE_OBJECT_ONLY, NULL);
+  } else if (enforce_arity(arity, 1)) {
+    if (state.debug_info) {
+      p = HEAD(parms);
+      if (p->tag == string) {
+        state.obj.debug_file = coff_add_filesym(p->value.string, 0);
+      } else {
+        gperror(GPE_ILLEGAL_ARGU, NULL);
+      }
+    } else {
+      gpwarning(GPW_UNKNOWN, "directive ignored when debug info is disabled");
+    }
+  }
+
+  return r;
+}
+
 /* Filling constants is handled here. Filling instructions is handled
    in the parser. */ 
    
@@ -1194,6 +1487,28 @@ static gpasmVal do_idata(gpasmVal r,
   return r;
 }
 
+static gpasmVal do_ident(gpasmVal r,
+		         char *name,
+		         int arity,
+		         struct pnode *parms)
+{
+  struct pnode *p;
+  state.lst.line.linetype = dir;
+  
+  if (state.mode == absolute) {
+    gperror(GPE_OBJECT_ONLY, NULL);
+  } else if (enforce_arity(arity, 1)) {
+    p = HEAD(parms);
+    if (p->tag == string) {
+      coff_add_identsym(p->value.string);
+    } else {
+      gperror(GPE_ILLEGAL_ARGU, NULL);
+    }
+  }
+
+  return r;
+}
+
 static gpasmVal do_idlocs(gpasmVal r,
 		          char *name,
 		          int arity,
@@ -1226,9 +1541,9 @@ static gpasmVal do_idlocs(gpasmVal r,
     }
   }
 
-  if ((state.mode == relocatable) && (state.found_idlocs == 0)) {
+  if ((state.mode == relocatable) && (!state.found_idlocs)) {
     coff_new_section(".idlocs", idreg >> _16bit_core, STYP_ABS | STYP_TEXT);
-    state.found_idlocs = 1;
+    state.found_idlocs = true;
   }
 
   state.lst.config_address = idreg;
@@ -1379,6 +1694,29 @@ static gpasmVal do_include(gpasmVal r,
   return r;
 }
 
+static gpasmVal do_line(gpasmVal r,
+		       char *name,
+		       int arity,
+		       struct pnode *parms)
+{
+  struct pnode *p;
+
+  state.lst.line.linetype = dir;
+
+  if (state.mode == absolute) {
+    gperror(GPE_OBJECT_ONLY, NULL);
+  } else if (enforce_arity(arity, 1)) {
+    if (state.debug_info) {
+      p = HEAD(parms);
+      state.obj.debug_line = maybe_evaluate(p);
+    } else {
+      gpwarning(GPW_UNKNOWN, "directive ignored when debug info is disabled");
+    }
+  }
+
+  return r;
+}
+
 /************************************************************************
  * do_list - parse the LIST directive
  *
@@ -1403,7 +1741,7 @@ static gpasmVal do_list(gpasmVal r,
 {
   struct pnode *p;
 
-  state.lst.enabled = 1;
+  state.lst.enabled = true;
   state.lst.line.linetype = dir;
 
   for (; parms; parms = TAIL(parms)) {
@@ -1442,6 +1780,10 @@ static gpasmVal do_list(gpasmVal r,
 	} else if (strcasecmp(lhs, "p") == 0) {
 	  if (enforce_simple(p->value.binop.p1))
 	    select_processor(p->value.binop.p1->value.symbol);
+	} else if (strcasecmp(lhs, "pe") == 0) {
+          state.extended_pic16e = true;
+	  if (enforce_simple(p->value.binop.p1))
+	    select_processor(p->value.binop.p1->value.symbol);
 	} else if (strcasecmp(lhs, "r") == 0) {
 	  if (enforce_simple(p->value.binop.p1))
 	    select_radix(p->value.binop.p1->value.symbol);
@@ -1529,11 +1871,11 @@ static gpasmVal do_noexpand(gpasmVal r,
 		       struct pnode *parms)
 {
   state.lst.line.linetype = dir;
-  if (state.cmd_line.macro_expand == 1){
+  if (state.cmd_line.macro_expand) {
     gpmessage(GPM_SUPLIN, NULL);
   } else {
     if (enforce_arity(arity, 0)) {
-      state.lst.expand = 0;
+      state.lst.expand = false;
     }
   }
   return r;
@@ -1547,7 +1889,7 @@ static gpasmVal do_nolist(gpasmVal r,
   state.lst.line.linetype = dir;
   
   if (!state.lst.force)
-    state.lst.enabled = 0;
+    state.lst.enabled = false;
     
   coff_add_nolistsym();
   
@@ -1679,18 +2021,13 @@ static gpasmVal do_pagesel(gpasmVal r,
   if (enforce_arity(arity, 1)) {
     p = HEAD(parms);
     if (state.mode == absolute) {
-      if (p->tag != symbol) {
-        gperror(GPE_ILLEGAL_LABEL, NULL);
-      } else {
-        address = maybe_evaluate(p);
-        page = gp_processor_check_page(state.device.class, address);
-
-        state.org += gp_processor_set_page(state.device.class, 
-                                           state.processor_info->num_pages,
-                                           page, 
-                                           state.i_memory, 
-                                           state.org);
-      }
+      address = maybe_evaluate(p);
+      page = gp_processor_check_page(state.device.class, address);
+      state.org += gp_processor_set_page(state.device.class, 
+                                         state.processor_info->num_pages,
+                                         page, 
+                                         state.i_memory, 
+                                         state.org);
     } else {
       num_reloc = count_reloc(p);
       
@@ -1851,6 +2188,43 @@ static gpasmVal do_space(gpasmVal r,
   return r;
 }
 
+static gpasmVal do_type(gpasmVal r,
+		        char *name,
+		        int arity,
+		        struct pnode *parms)
+{
+  struct pnode *p;
+  char *symbol_name = NULL;
+  gp_symbol_type *coff_symbol = NULL;
+  int value;
+
+  state.lst.line.linetype = dir;
+  
+  if (state.mode == absolute) {
+    gperror(GPE_OBJECT_ONLY, NULL);
+  } else if (enforce_arity(arity, 2)) {
+    /* the first argument is the symbol name */
+    p = HEAD(parms);
+    if (enforce_simple(p)) {
+      symbol_name = p->value.symbol;
+      coff_symbol = gp_coffgen_findsymbol(state.obj.object, symbol_name);
+      if (coff_symbol == NULL) {
+        gperror(GPE_NOSYM, NULL);
+      } else {
+        p = HEAD(TAIL(parms));
+        value = maybe_evaluate(p);   
+        if ((value < 0) || (value > 0xffff)) {
+          gperror(GPE_RANGE, NULL);
+        } else {
+          coff_symbol->type = value;   
+        }
+      }
+    }
+  }
+
+  return r;
+}
+
 static gpasmVal do_udata(gpasmVal r,
 		         char *name,
 		         int arity,
@@ -2252,6 +2626,13 @@ gpasmVal do_insn(char *name, struct pnode *parms)
 	  emit_check(i->opcode, (reloc_evaluate(p, RELOCT_MOVLR) << 4), 0xf0);
 	}
 	break;
+      case INSN_CLASS_LIT6:
+	if (enforce_arity(arity, 1)) {
+	  p = HEAD(parms);
+	  /* The literal cannot be a relocatable address */
+          emit_check(i->opcode, maybe_evaluate(p), 0x3f);
+	}
+	break;
       case INSN_CLASS_LIT8:
 	if (enforce_arity(arity, 1)) {
 	  p = HEAD(parms);
@@ -2355,6 +2736,25 @@ gpasmVal do_insn(char *name, struct pnode *parms)
 	  emit(i->opcode | (value & 0x1fff));
 	}
 	break;
+      case INSN_CLASS_LITFSR:
+	if (enforce_arity(arity, 2)) {
+	  int value;
+          int fsr;
+
+	  p = HEAD(parms);
+          fsr = maybe_evaluate(p);
+	  if ((fsr < 0) || (fsr > 2))
+            gperror(GPE_RANGE, NULL); 	  
+
+          p = HEAD(TAIL(parms));
+	  /* the offset cannot be a relocatable address */
+          value = maybe_evaluate(p);
+	  if (value & (~0x3f))
+            gperror(GPE_RANGE, NULL); 	  
+
+	  emit(i->opcode | ((fsr & 0x3) << 6) | (value & 0x3f));
+	}
+	break;
       case INSN_CLASS_RBRA8:
 	if (enforce_arity(arity, 1)) {
 	  int offset;
@@ -2488,6 +2888,50 @@ gpasmVal do_insn(char *name, struct pnode *parms)
               (file & 0xff) );
         }
         break;
+      case INSN_CLASS_SF:
+        if (enforce_arity(arity, 2)) {
+          int source;
+	  int dest;
+
+	  p = HEAD(parms);
+          if (p->tag != offset)
+	    gperror(GPE_MISSING_BRACKET, NULL);
+          source = maybe_evaluate(p);
+
+          p = HEAD(TAIL(parms));
+          dest = maybe_evaluate(p);
+
+          /* destination can't be PCL, TOSU, TOSH, TOSL */
+          if ((dest == 0xff9) ||
+              (dest == 0xfff) ||
+              (dest == 0xffe) ||
+              (dest == 0xffd)) {
+            gperror(GPE_UNKNOWN, "Invalid destination");
+          }
+          
+	  emit_check(i->opcode, source, 0x7f);
+	  emit_check(0xf000, reloc_evaluate(HEAD(TAIL(parms)), RELOCT_FF2), 0xfff);
+        }
+        break;
+      case INSN_CLASS_SS:
+        if (enforce_arity(arity, 2)) {
+          int source;
+	  int dest;
+
+	  p = HEAD(parms);
+          if (p->tag != offset)
+	    gperror(GPE_MISSING_BRACKET, NULL);
+          source = maybe_evaluate(p);
+
+          p = HEAD(TAIL(parms));
+          if (p->tag != offset)
+	    gperror(GPE_MISSING_BRACKET, NULL);
+          dest = maybe_evaluate(p);
+
+	  emit_check(i->opcode, source, 0x7f);
+	  emit_check(0xf000, dest, 0x7f);
+        }
+        break;
       case INSN_CLASS_OPF5:
 	if (enforce_arity(arity, 1)) {
 	  p = HEAD(parms);
@@ -2987,7 +3431,15 @@ struct insn op_0[] = {
   { "udata_shr",  0, (long int)do_udata_shr, INSN_CLASS_FUNC,   0 },
   { "variable",   0, (long int)do_variable,  INSN_CLASS_FUNC,   0 },
   { "while",      0, (long int)do_while,     INSN_CLASS_FUNC,   0 },
+  { ".def",       0, (long int)do_def, 	     INSN_CLASS_FUNC,   0 },
+  { ".dim",       0, (long int)do_dim, 	     INSN_CLASS_FUNC,   0 },
+  { ".direct",    0, (long int)do_direct,    INSN_CLASS_FUNC,   0 },
+  { ".eof",       0, (long int)do_eof,	     INSN_CLASS_FUNC,   0 },
+  { ".file",      0, (long int)do_file,	     INSN_CLASS_FUNC,   0 },
+  { ".ident",     0, (long int)do_ident,     INSN_CLASS_FUNC,   0 },
+  { ".line",      0, (long int)do_line,	     INSN_CLASS_FUNC,   0 },
   { ".set",       0, (long int)do_set, 	     INSN_CLASS_FUNC,   0 },
+  { ".type",      0, (long int)do_type,	     INSN_CLASS_FUNC,   0 },
   { "#if",        0, (long int)do_if,        INSN_CLASS_FUNC,   ATTRIB_COND },
   { "#else",      0, (long int)do_else,	     INSN_CLASS_FUNC,   ATTRIB_COND },
   { "#endif",     0, (long int)do_endif,     INSN_CLASS_FUNC,   ATTRIB_COND },
@@ -3097,6 +3549,13 @@ void opcode_init(int stage)
 	annotate_symbol( add_symbol(state.stBuiltin, op_18cxx_sp[i].name), 
 			 &op_18cxx_sp[i]);
 
+      if (state.extended_pic16e) {
+        /* Some 18xx devices have an extended instruction set. */
+        for (i = 0; i < num_op_18cxx_ext; i++)
+	  annotate_symbol( add_symbol(state.stBuiltin, op_18cxx_ext[i].name), 
+                           &op_18cxx_ext[i]);
+      }
+
       break;
 
     default:
