@@ -1,5 +1,5 @@
 /* Error handling for gpasm
-   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004
+   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005
    James Bowman, Craig Franklin
 
 This file is part of gputils.
@@ -153,6 +153,8 @@ char *gp_geterror(unsigned int code)
     return "__CONFIG directives must be contiguous.";
   case GPE_CONTIG_IDLOC:
     return "__IDLOC directives must be contiguous.";
+  case GPE_MISSING_BRACKET:
+    return "Square brackets required around offset operand.";
   case GPE_UNKNOWN:
   default:
     return "UNKNOWN";
