@@ -1,5 +1,5 @@
 /* Error handling
-   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004 
+   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005
    James Bowman, Craig Franklin
 
 This file is part of gputils.
@@ -86,7 +86,8 @@ enum GPE_codes {
   GPE_UNKNOWN          = 157,
   GPE_FILL_ODD         = 159,
   GPE_CONTIG_CONFIG    = 163,
-  GPE_CONTIG_IDLOC     = 164
+  GPE_CONTIG_IDLOC     = 164,
+  GPE_MISSING_BRACKET  = 168
 };
 
 /* Warning codes
