@@ -1,5 +1,5 @@
 /* Disassembles ".HEX" files
-   Copyright (C) 2001, 2002, 2003, 2004
+   Copyright (C) 2001, 2002, 2003, 2004, 2005
    Craig Franklin
 
 This file is part of gputils.
@@ -139,12 +139,14 @@ void dasm(MemBlock *memory)
                                    sizeof(buffer));
         printf("%s\n", buffer);
         i++;
-        if ((state.format) && (num_words != 1)) {
+        if (num_words != 1) {
           /* some 18xx instructions use two words */
-          printf("%06x:  %04x\n",
-                 i << byte_addr,
-                 (i_memory_get(memory, i) & 0xffff));
-          i++;
+	  if (state.format) {
+	    printf("%06x:  %04x\n",
+                   i << byte_addr,
+                   (i_memory_get(memory, i) & 0xffff));
+          }
+	  i++;
         }        
       } 
     }
@@ -160,6 +162,7 @@ void show_usage(void)
 {
   printf("Usage: gpdasm [options] file\n");
   printf("Options: [defaults in brackets after descriptions]\n");
+  printf("  -c, --mnemonics                Decode special mnemonics.\n");
   printf("  -h, --help                     Show this usage message.\n");
   printf("  -i, --hex-info                 Information on input hex file.\n");
   printf("  -l, --list-chips               List supported processors.\n");
@@ -167,17 +170,19 @@ void show_usage(void)
   printf("  -p PROC, --processor PROC      Select processor.\n");
   printf("  -s, --short                    Print short format.\n");
   printf("  -v, --version                  Show version.\n");
+  printf("  -y, --extended                 Enable 18xx extended mode.\n");
   printf("\n");
   printf("Report bugs to:\n");
-  printf("%s\n", BUG_REPORT_URL);
+  printf("%s\n", PACKAGE_BUGREPORT);
   exit(0);
 }
 
-#define GET_OPTIONS "?hilmp:sv"
+#define GET_OPTIONS "?chilmp:svy"
 
   /* Used: himpsv */
   static struct option longopts[] =
   {
+    { "mnemonics",   0, 0, 'c' },
     { "help",        0, 0, 'h' },
     { "hex-info",    0, 0, 'i' },
     { "list-chips",  0, 0, 'l' },
@@ -185,6 +190,7 @@ void show_usage(void)
     { "processor",   1, 0, 'p' },
     { "short",       0, 0, 's' },
     { "version",     0, 0, 'v' },
+    { "extended",    0, 0, 'y' },
     { 0, 0, 0, 0 }
   };
 
@@ -210,6 +216,9 @@ int main(int argc, char *argv[])
     case 'h':
       usage = 1;
       break;
+    case 'c':
+      gp_decode_mnemonics = true;
+      break;
     case 'i':
       print_hex_info = 1;
       break;
@@ -226,6 +235,9 @@ int main(int argc, char *argv[])
     case 's':
       state.format = 0;
       break;
+    case 'y':
+      gp_decode_extended = true;
+      break;
     case 'v':
       fprintf(stderr, "%s\n", GPDASM_VERSION_STRING);
       exit(0);
