@@ -1,5 +1,5 @@
 /* GNU PIC view object
-   Copyright (C) 2001, 2002, 2003, 2004
+   Copyright (C) 2001, 2002, 2003, 2004, 2005
    Craig Franklin
 
 This file is part of gputils.
@@ -230,6 +230,7 @@ void print_sym_table (gp_object_type *object)
   gp_aux_type *aux;
   char *section;
   int i;
+  char c;
 
   symbol = object->symbols;
 
@@ -261,6 +262,12 @@ void print_sym_table (gp_object_type *object)
       aux = symbol->aux_list;
       
       switch (aux->type) {
+      case AUX_DIRECT:
+        printf("  command = \"%c\"\n", 
+               aux->_aux_symbol._aux_direct.command);
+        printf("  string = \"%s\"\n", 
+               aux->_aux_symbol._aux_direct.string);
+        break;
       case AUX_FILE:
         if (!state.suppress_names) {
           printf("  file = %s\n", 
@@ -269,6 +276,10 @@ void print_sym_table (gp_object_type *object)
         printf("  line included = %li\n", 
                aux->_aux_symbol._aux_file.line_number);
         break;
+      case AUX_IDENT:
+        printf("  string = \"%s\"\n", 
+               aux->_aux_symbol._aux_ident.string);
+        break;
       case AUX_SCN:
         printf("  length = %li\n", 
                aux->_aux_symbol._aux_scn.length);
@@ -279,8 +290,16 @@ void print_sym_table (gp_object_type *object)
         break;
       default:
         printf("  ");
-        for (i = 0; i < SYMBOL_SIZE; i++)
-          printf("%2x", aux->_aux_symbol.data[i]);
+        for (i = 0; i < SYMBOL_SIZE; i++) {
+          printf("%02x", aux->_aux_symbol.data[i]);
+          if (i & 1) {
+            printf(" ");
+          }
+        }
+        for (i = 0; i < SYMBOL_SIZE; i++) {
+          c = aux->_aux_symbol.data[i];
+          putchar( (isprint(c)) ? c : '.');
+        }
       }
     }
 
@@ -339,30 +358,34 @@ void show_usage(void)
   printf("Usage: gpvo [options] file\n");
   printf("Options: [defaults in brackets after descriptions]\n");
   printf("  -b, --binary               Print binary data.\n");
+  printf("  -c, --mnemonics            Decode special mnemonics.\n");
   printf("  -f, --file                 File header.\n");
   printf("  -h, --help                 Show this usage message.\n");
   printf("  -n, --no-names             Suppress filenames.\n");
   printf("  -s, --section              Section data.\n");
   printf("  -t  --symbol               Symbol table.\n");
   printf("  -v, --version              Show version.\n");
+  printf("  -y, --extended             Enable 18xx extended mode.\n");
   printf("\n");
   printf("Report bugs to:\n");
-  printf("%s\n", BUG_REPORT_URL);
+  printf("%s\n", PACKAGE_BUGREPORT);
   exit(0);
 }
 
-#define GET_OPTIONS "?bfhnstv"
+#define GET_OPTIONS "?bcfhnstvy"
 
   /* Used: himpsv */
   static struct option longopts[] =
   {
     { "binary",      0, 0, 'b' },
+    { "mnemonics",   0, 0, 'c' },
     { "file",        0, 0, 'f' },
     { "help",        0, 0, 'h' },
     { "no-names",    0, 0, 'n' },
     { "section",     0, 0, 's' },
     { "symbol",      0, 0, 't' },
     { "version",     0, 0, 'v' },
+    { "extended",    0, 0, 'y' },
     { 0, 0, 0, 0 }
   };
 
@@ -377,9 +400,8 @@ int main(int argc, char *argv[])
   gp_init();
 
   /* initalize */
-  state.quiet = 0;
   state.dump_flags = 0;
-  state.suppress_names = 0;
+  state.suppress_names = false;
 
   while ((c = GETOPT_FUNC) != EOF) {
     switch (c) {
@@ -390,11 +412,14 @@ int main(int argc, char *argv[])
     case 'b':
       state.dump_flags |= PRINT_BINARY;
       break;
+    case 'c':
+      gp_decode_mnemonics = true;
+      break;
     case 'f':
       state.dump_flags |= PRINT_HEADER;
       break;
     case 'n':
-      state.suppress_names = 1;
+      state.suppress_names = true;
       break;
     case 's':
       state.dump_flags |= PRINT_SECTIONS;
@@ -402,6 +427,9 @@ int main(int argc, char *argv[])
     case 't':
       state.dump_flags |= PRINT_SYMTBL;
       break;
+    case 'y':
+      gp_decode_extended = true;
+      break;
     case 'v':
       fprintf(stderr, "%s\n", GPVO_VERSION_STRING);
       exit(0);
