@@ -1,5 +1,5 @@
 /* Disassemble memory
-   Copyright (C) 2001, 2002, 2003, 2004
+   Copyright (C) 2001, 2002, 2003, 2004, 2005
    Craig Franklin
 
 This file is part of gputils.
@@ -44,6 +44,9 @@ Boston, MA 02111-1307, USA.  */
                                              ARG2, \
                                              ARG3)
 
+gp_boolean gp_decode_mnemonics = false;
+gp_boolean gp_decode_extended = false;
+
 int
 gp_disassemble(MemBlock *m,
                int org,
@@ -97,10 +100,29 @@ gp_disassemble(MemBlock *m,
     }
     break;
   case PROC_CLASS_PIC16E:
-    for(i = 0; i < num_op_18cxx; i++) {
-      if((op_18cxx[i].mask & opcode) == op_18cxx[i].opcode) {
-        instruction = &op_18cxx[i];
-        break;
+    if (gp_decode_mnemonics) {
+      for(i = 0; i < num_op_18cxx_sp; i++) {
+        if((op_18cxx_sp[i].mask & opcode) == op_18cxx_sp[i].opcode) {
+          instruction = &op_18cxx_sp[i];
+          break;
+        }
+      }
+    }
+    if (instruction == NULL) {
+      for(i = 0; i < num_op_18cxx; i++) {
+        if((op_18cxx[i].mask & opcode) == op_18cxx[i].opcode) {
+          instruction = &op_18cxx[i];
+          break;
+        }
+      }
+    }
+    if ((instruction == NULL) && (gp_decode_extended)) {
+      /* might be from the extended instruction set */
+      for(i = 0; i < num_op_18cxx_ext; i++) {
+        if((op_18cxx_ext[i].mask & opcode) == op_18cxx_ext[i].opcode) {
+          instruction = &op_18cxx_ext[i];
+          break;
+        }
       }
     }
     break;
@@ -130,6 +152,9 @@ gp_disassemble(MemBlock *m,
     case INSN_CLASS_LIT4S:
       DECODE_ARG1((opcode & 0xf0) >> 4); 
       break;
+    case INSN_CLASS_LIT6:
+      DECODE_ARG1(opcode & 0x3f); 
+      break;
     case INSN_CLASS_LIT8:
     case INSN_CLASS_LIT8C12:
     case INSN_CLASS_LIT8C16:
@@ -144,6 +169,9 @@ gp_disassemble(MemBlock *m,
     case INSN_CLASS_LIT13:
       DECODE_ARG1(opcode & 0x1fff); 
       break;
+    case INSN_CLASS_LITFSR:
+      DECODE_ARG2(((opcode >> 6) & 0x3), (opcode & 0x3f));
+      break;
     case INSN_CLASS_RBRA8:
       value = opcode & 0xff;
       /* twos complement number */
@@ -165,7 +193,7 @@ gp_disassemble(MemBlock *m,
         long int dest;
 
         num_words = 2;
-        dest = (i_memory_get(m, org) & 0xfff) << 8;
+        dest = (i_memory_get(m, org + 1) & 0xfff) << 8;
         dest |= opcode & 0xff;      
         DECODE_ARG1(dest * 2); 
       }
@@ -175,7 +203,7 @@ gp_disassemble(MemBlock *m,
         long int dest;
 
         num_words = 2;
-        dest = (i_memory_get(m, org) & 0xfff) << 8;
+        dest = (i_memory_get(m, org + 1) & 0xfff) << 8;
         dest |= opcode & 0xff;      
 	snprintf(buffer, sizeof_buffer, "%s\t%#lx, %#lx",
                 instruction->name,
@@ -189,7 +217,7 @@ gp_disassemble(MemBlock *m,
         long int file;
 
         num_words = 2;
-        k = i_memory_get(m, org) & 0xff;
+        k = i_memory_get(m, org + 1) & 0xff;
         k |= ((opcode & 0xf) << 8);
 	file = (opcode >> 4) & 0x3;
         DECODE_ARG2(file, k);
@@ -202,7 +230,7 @@ gp_disassemble(MemBlock *m,
 
         num_words = 2;
         file1 = opcode & 0xfff;
-        file2 = i_memory_get(m, org) & 0xfff;
+        file2 = i_memory_get(m, org + 1) & 0xfff;
         DECODE_ARG2(file1, file2);
       }
       break;
@@ -212,6 +240,28 @@ gp_disassemble(MemBlock *m,
     case INSN_CLASS_PF:
       DECODE_ARG2(((opcode >> 8) & 0x1f), (opcode & 0xff));
       break;
+    case INSN_CLASS_SF:
+      {
+        long int offset;
+        long int file;
+
+        num_words = 2;
+        offset = opcode & 0x7f;
+        file = i_memory_get(m, org + 1) & 0xfff;
+        DECODE_ARG2(offset, file);
+      }
+      break;
+    case INSN_CLASS_SS:
+      {
+        long int offset1;
+        long int offset2;
+
+        num_words = 2;
+        offset1 = opcode & 0x7f;
+        offset2 = i_memory_get(m, org + 1) & 0x7f;
+        DECODE_ARG2(offset1, offset2);
+      }
+      break;
     case INSN_CLASS_OPF5:
       DECODE_ARG1(opcode & 0x1f);
       break;
