@@ -1,5 +1,5 @@
 /* Read coff objects 
-   Copyright (C) 2001, 2002, 2003, 2004
+   Copyright (C) 2001, 2002, 2003, 2004, 2005
    Craig Franklin
 
 This file is part of gputils.
@@ -303,11 +303,20 @@ _read_aux(gp_aux_type *aux, int aux_type, char *file, char *string_table)
   aux->type = aux_type;
 
   switch (aux_type) {
+    case AUX_DIRECT:
+      aux->_aux_symbol._aux_direct.command = file[0];
+      aux->_aux_symbol._aux_direct.string = 
+        strdup(&string_table[gp_getl32(&file[4])]);
+      break;
     case AUX_FILE:
       aux->_aux_symbol._aux_file.filename = 
         strdup(&string_table[gp_getl32(&file[0])]);
       aux->_aux_symbol._aux_file.line_number = gp_getl32(&file[4]);
       break;
+    case AUX_IDENT:
+      aux->_aux_symbol._aux_ident.string = 
+        strdup(&string_table[gp_getl32(&file[0])]);
+      break;
     case AUX_SCN:
       aux->_aux_symbol._aux_scn.length  = gp_getl32(&file[0]);
       aux->_aux_symbol._aux_scn.nreloc  = gp_getl16(&file[4]);
@@ -390,10 +399,10 @@ _read_symtbl(gp_object_type *object, char *file)
 
         /* advance the through the list */
         for (j = 0; j < num_auxsym; j++) {
-          /* COFF places all symbols inluding auxilary, in the symbol table.
-             However, in memory, gputils attaches auxilary symbols to their
+          /* COFF places all symbols inluding auxiliary, in the symbol table.
+             However, in memory, gputils attaches auxiliary symbols to their
              associated primary symbol.  When reading COFF, space is reserved
-             for the auxilary symbols but not used.  Later the space is 
+             for the auxiliary symbols but not used.  Later the space is 
              freed.  This simplifies assigning the pointer in the 
              relocations. */
           current = current->next;
@@ -406,7 +415,7 @@ _read_symtbl(gp_object_type *object, char *file)
   }
 }
 
-/* remove space reserved for auxilary entries, add section pointers, and 
+/* remove space reserved for auxiliary entries, add section pointers, and 
    setup tail pointer */
 
 static void 
