@@ -1,5 +1,5 @@
 /* Symbol table support
-   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004
+   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005
    James Bowman
 
 This file is part of gputils.
@@ -53,7 +53,7 @@ static int hashfunc(struct symbol_table *t, char *s)
 }
 
 struct symbol_table *push_symbol_table(struct symbol_table * table,
-				       int case_insensitive)
+				       gp_boolean case_insensitive)
 {
   struct symbol_table *new = calloc(sizeof(*new), 1);
 
