@@ -1,5 +1,5 @@
 /* Write coff objects 
-   Copyright (C) 2003, 2004
+   Copyright (C) 2003, 2004, 2005
    Craig Franklin
 
 This file is part of gputils.
@@ -217,7 +217,7 @@ _gp_coffgen_write_linenum(gp_section_type *section, FILE *fp)
   return;
 }
 
-/* write the auxilary symbols */
+/* write the auxiliary symbols */
 static void 
 _gp_coffgen_write_auxsymbols(gp_aux_type *aux, char *table, FILE *fp) 
 {
@@ -226,6 +226,16 @@ _gp_coffgen_write_auxsymbols(gp_aux_type *aux, char *table, FILE *fp)
   while(aux != NULL) {
 
     switch (aux->type) {
+    case AUX_DIRECT:
+      /* add the direct string to the string table */
+      offset = _gp_coffgen_addstring(aux->_aux_symbol._aux_direct.string, 
+                                     table);
+      gp_fputl32(aux->_aux_symbol._aux_direct.command, fp);
+      gp_fputl32(offset, fp);
+      gp_fputl32(0, fp);
+      gp_fputl32(0, fp);
+      gp_fputl16(0, fp);
+      break;
     case AUX_FILE:
       /* add the filename to the string table */
       offset = _gp_coffgen_addstring(aux->_aux_symbol._aux_file.filename, 
@@ -236,8 +246,18 @@ _gp_coffgen_write_auxsymbols(gp_aux_type *aux, char *table, FILE *fp)
       gp_fputl32(0, fp);
       gp_fputl16(0, fp);
       break;
+    case AUX_IDENT:
+      /* add the ident string to the string table */
+      offset = _gp_coffgen_addstring(aux->_aux_symbol._aux_ident.string, 
+                                     table);
+      gp_fputl32(offset, fp);
+      gp_fputl32(0, fp);
+      gp_fputl32(0, fp);
+      gp_fputl32(0, fp);
+      gp_fputl16(0, fp);
+      break;
     case AUX_SCN:
-      /* write section auxilary symbol */
+      /* write section auxiliary symbol */
       gp_fputl32(aux->_aux_symbol._aux_scn.length, fp);
       gp_fputl16(aux->_aux_symbol._aux_scn.nreloc, fp);
       gp_fputl16(aux->_aux_symbol._aux_scn.nlineno, fp);
